<?php

namespace Drupal\jsonapi_section_components\Resource;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityBase;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi\ResourceResponse;
use Drupal\jsonapi_resources\Resource\EntityQueryResourceBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi_section_components\Plugin\Block\SectionComponentBlockInterface;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Drupal\layout_builder\SectionStorage\SectionStorageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes a request for a collection containing featured articles nodes.
 */
class LayoutBuilderTemstoreResource extends EntityQueryResourceBase implements ContainerInjectionInterface {

  /**
   * Constructs a new LayoutBuilderTemstoreResource object.
   *
   * @param \Drupal\layout_builder\SectionStorage\SectionStorageManagerInterface $section_storage_manager
   *   The section storage manager.
   * @param \Drupal\layout_builder\LayoutTempstoreRepositoryInterface $layout_builder_tempstore_repository
   *   The layout builder tempstore repository.
   * @param \Drupal\Core\Block\BlockManagerInterface $block_manager
   *   The block manager.
   */
  public function __construct(
    protected SectionStorageManagerInterface $sectionStorageManager,
    protected LayoutTempstoreRepositoryInterface $layoutBuilderTempstoreRepository,
    protected BlockManagerInterface $blockManager,
  ) {

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.layout_builder.section_storage'),
      $container->get('layout_builder.tempstore_repository'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * Process the resource request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The response.
   */
  public function process(Request $request, EntityBase $content, string $view_mode, string $component): ResourceResponse {
    $cacheability = new CacheableMetadata();
    $cacheability->setCacheMaxAge(0);

    $definition = [
      'layout_builder_tempstore' => TRUE,
      'converter' => 'layout_builder.param_converter',
    ];
    $entity_type_id = $content->getEntityTypeId();
    $defaults = [
      'entity_type_id' => $entity_type_id,
      'section_storage_type' => 'overrides',
      '_entity_form' => $entity_type_id . '.layout_builder',
      $entity_type_id => $content->id(),
      '_route' => 'layout_builder.overrides.' . $entity_type_id . '.view',
    ];
    $type = 'overrides';
    $name = "section_storage";
    $context = $this->sectionStorageManager->loadEmpty($type)->deriveContextsFromRoute('', $definition, $name, $defaults);
    $section_storage = $this->sectionStorageManager->load($type, $context);
    $ss = $this->layoutBuilderTempstoreRepository->get($section_storage);
    $sections = $ss->getSections();
    $block_instance = NULL;
    foreach ($sections as $section) {
      /** @var Drupal\layout_builder\SectionComponent $section_component */
      foreach ($section->getComponents() as $section_component) {
        if ($section_component->getUuid() === $component) {
          $config = $section_component->get('configuration');
          $block = $this->blockManager->createInstance($config['id'], $config);
          if ($block instanceof SectionComponentBlockInterface) {
            $block_instance = $block;
            break 2;
          }
        }
      }
    }

    if ($block_instance === NULL) {
      throw new \Exception('Block instance of type SectionComponentBlockInterface not found.');
    }

    /** @var \Drupal\Core\Entity\EntityBase $response_entity */
    $response_entity = $block_instance->getResponseEntity($section_component, $content);
    $ro = ResourceObject::createFromEntity($this->resourceTypeRepository->get($response_entity->getEntityTypeId(), $response_entity->bundle()), $response_entity);
    $ro_data = new ResourceObjectData([$ro]);

    $meta = [
      'section_component' => $block_instance->processSectionComponent($section_component),
    ];

    /** @var \Drupal\jsonapi\CacheableResourceResponse $response */
    $response = $this->createJsonapiResponse($ro_data, $request, 200, [], NULL, $meta);
    $response->addCacheableDependency($cacheability);

    return $response;
  }

}
