<?php

namespace Drupal\jsonapi_section_components\Routing;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\Routing\Route;

/**
 * Defines dynamic routes.
 */
class Resources implements ContainerInjectionInterface {

  /**
   * Constructs a new AutomatorRouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_display.repository'),
    );
  }

  /**
   * Provides dynamic routes.
   */
  public function routes() {
    $routes = [];
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type) {
      if (!is_subclass_of($entity_type->getClass(), ContentEntityInterface::class)) {
        continue;
      }
      $resource_types = $this->getBlockResourceTypes();
      $bundles = array_keys($this->entityTypeBundleInfo->getBundleInfo($entity_type->id()));
      $view_modes = array_keys($this->entityDisplayRepository->getViewModes($entity_type->id()));
      $view_modes[] = 'default';
      foreach ($bundles as $bundle) {
        $has_layouts = FALSE;
        foreach ($view_modes as $view_mode_id) {
          /** @var Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay $display */
          $display = $this->entityDisplayRepository->getViewDisplay($entity_type->id(), $bundle, $view_mode_id);
          if ($display && !$display->isOverridable()) {
            continue;
          }
          $has_layouts = TRUE;
          break;
        }
        if ($has_layouts) {
          $resource_types[] = $entity_type->id() . '--' . $bundle;
        }
      }
      if (!empty($resource_types)) {
        $route = $this->buildRoute($entity_type->id(), $resource_types);
        $routes['jsonapi_section_components.jsonapi_resources.' . $entity_type->id()] = $route;
      }
    }
    return $routes;
  }

  /**
   * Returns the resource types for block content.
   *
   * @return array
   *   An array of resource types.
   */
  protected function getBlockResourceTypes() {
    $bundles = array_keys($this->entityTypeBundleInfo->getBundleInfo('block_content'));
    $resource_types = [];
    foreach ($bundles as $bundle) {
      $resource_types[] = 'block_content--' . $bundle;
    }
    return $resource_types;
  }

  /**
   * Builds a route for the given entity type, bundle, and view mode.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param array $resource_types
   *   The type of resources.
   *
   * @return \Symfony\Component\Routing\Route|null
   *   The route object or NULL if the route cannot be built.
   */
  protected function buildRoute(string $entity_type_id, array $resource_types): Route {
    return new Route(
      // Path to attach this route to:
      '/%jsonapi%/section_component/' . $entity_type_id . '/{content}/{view_mode}/{component}',
      // Route defaults:
      [
        '_jsonapi_resource' => 'Drupal\jsonapi_section_components\Resource\LayoutBuilderTemstoreResource',
        '_jsonapi_resource_types' => $resource_types,
      ],
      // Route requirements:
      [
        '_permission'  => 'access content',
      ],
      [
        'parameters' => [
          'content' => [
            'type' => 'entity:' . $entity_type_id,
            'converter' => 'paramconverter.jsonapi.entity_uuid',
          ],
          'view_mode' => [
            'type' => 'string',
          ],
          'component' => [
            'type' => 'string',
          ],
        ],
        'no_cache' => 'TRUE',
      ]
    );
  }

}
